import pygame,math,random
import os.path as path

from cnst import*

from animate import*
from particle import*
from projectile import*

class Player(object):
	
	def __init__(self,game,x,y):
	
		self.game = game
		self.x = x
		self.y = y
		
		left_anim = Animation(path.join(IMGDIR,"guy_left.png"),16,1)
		stand_anim = Animation(path.join(IMGDIR,"guy_stand.png"),16,1)
		right_anim = Animation(path.join(IMGDIR,"guy_right.png"),16,1)
		
		self.anims = [stand_anim,left_anim,right_anim]
		self.anim_id = 0
		
		self.cross_image = pygame.image.load(path.join(IMGDIR,"hair.png")).convert_alpha()
		
		#self.anim = Animation("images\\henge_rn_rt.png",36,1)
		#self.anim.play()
		self.image = self.anims[0].get_image()
		self.gun = Gun(self,self.x,self.y) 
		self.dash_speed = 2.6
		self.max_dashtime = 700
		self.is_dashing = False
		self.dash_acctime = 0
		self.g_img = pygame.image.load(path.join(IMGDIR,"gun.png"))
		
		self.gun_image = self.g_img.copy()
		self.gun_rect = self.gun_image.get_rect()
		self.gun_rot_rect = self.gun_image.get_rect()		
		
		self.dash_wait = 300
		self.dash_wctime = 0 
		
		self.prev_positions = []
		
		self.trail_len = 10
		
		#self.emitter = Emitter(path.join(IMGDIR,"smoke.png"),12,self.x,self.y,
		#							friction = 0.97,fps=6,angle_variance = 10, kind = 2)
		#self.emitter.start_stream()
		
		self.xvel = 0
		self.yvel = 0
		self.speed = 0.09
		self.friction = 0.95
		self.alive = True
		
		self.rect = pygame.Rect(x,y,16,16)
		
		self.mu = False
		self.md = False
		self.mr = False
		self.ml = False	
		
		self.cross_rect = pygame.Rect(320/2,240/2,10,10) 
	

	def handle_collisions(self):
		
		for block in self.game.block_objects:
			
			if self.rect.colliderect(block.rect):
				
				# check penetration
				xpen = ((self.rect.w/2) + (block.rect.w/2)) - abs(block.rect.centerx - self.rect.centerx) 
				ypen = ((self.rect.h/2) + (block.rect.h/2)) - abs(block.rect.centery - self.rect.centery) 
				
				pen_dir = min(xpen,ypen)
				#print xpen,ypen
				
				if pen_dir == xpen:
					
					if ypen > 2:
					
						if self.rect.centerx <= block.rect.centerx:
							# move left
							self.x = block.rect.left - (self.rect.w - 1)
							if self.xvel > 0:
								self.xvel = 0.0
			
						else:
							self.x = block.rect.right - 1
							if self.xvel < 0:
								self.xvel = 0.0
									
				else:
					if self.rect.centery <= block.rect.centery:
						# move up
						self.y = block.rect.top - (self.rect.h-1)
						
						if self.yvel > 0:
							self.yvel = 0.0
		
					else:
						self.y = block.rect.bottom  
						
						#print "yrll"
						if self.yvel < 0:
							self.yvel = 0.0
				
	
	def delimit(self):
		
		if self.x < 0:
			self.x = 0
			self.xvel *= -1
			
		if self.x > 320 - self.rect.w:
			self.x = 320 - self.rect.w
			self.xvel *= -1
			
		if self.y < 0:
			self.y = 0
			self.yvel *= -1
			
		if self.y > 240 - self.rect.h:
			self.y = 240 - self.rect.h
			self.yvel *= -1
		

	def move(self):
		
		if self.ml:
			self.xvel -= self.speed
			self.anim_id = 1
			self.anims[self.anim_id].play()
			
		elif self.mr:
			self.xvel += self.speed
			self.anim_id = 2
			self.anims[self.anim_id].play()
			
		if self.mu:
			self.yvel -= self.speed
		
		elif self.md:
			self.yvel += self.speed
	
	def set_cursor_pos(self,x,y):
		self.cross_rect.center = (x,y)
	
	
	def dash(self):
		
		if self.dash_wctime > self.dash_wait:
				
			if self.ml:
				self.xvel -= self.dash_speed
				self.is_dashing = True
				self.dash_acctime = 0
				self.dash_wctime = 0
				
			elif self.mr:
				self.xvel += self.dash_speed
				self.is_dashing = True
				self.dash_acctime = 0
				self.dash_wctime = 0
				
			elif self.mu:
				self.yvel -= self.dash_speed
				self.is_dashing = True
				self.dash_acctime = 0
				self.dash_wctime = 0
				
			elif self.md:
				self.yvel += self.dash_speed
				self.is_dashing = True
				self.dash_acctime = 0
				self.dash_wctime = 0
			
			self.game.game_sounds["dash"].play()
				
	def draw(self,surface):
		
		#pygame.draw.rect(surface,(200,0,0),self.rect)
		#pygame.draw.rect(surface,(0,100,0),self.cross_rect)
		if self.alive:
			if self.is_dashing:
				pass
				
			################################################
			
			if len(self.prev_positions) > 1:
				
				if self.is_dashing:
					#pygame.draw.lines(surface,(200,200,0),0,self.prev_positions,16)	
					for pos in self.prev_positions:
						pygame.draw.rect(surface,(250,250,0),(pos[0]-8,pos[1]-8,16,16))
					
					
			surface.blit(self.image,self.rect)
				
			#self.emitter.draw(surface)	
			self.gun.draw(surface)
			surface.blit(self.gun_image,self.gun_rot_rect)
			surface.blit(self.cross_image,self.cross_rect)
		
			
	def test_hit(self):
		
		for enemy in self.game.traps:
			if enemy.rect.colliderect(self.rect):
				self.alive = False
				#self.game.is_running = False
				#pygame.time.wait(500)
				
				# add splat animation
				new_anim = Animation(path.join(IMGDIR,"splat.png"),64,1)
				new_anim.play(1)
				new_anim.set_fps(12)
				self.game.anims.append(new_anim)
				self.game.dead_pos = (self.rect.x - 16,self.rect.y - 16)
				self.game.game_sounds["over"].play()
				break
		
		for laser in self.game.lasers:
			if laser.rect.colliderect(self.rect):
				self.alive = False
				#self.game.is_running = False
				#pygame.time.wait(500)
				
				# add splat animation
				new_anim = Animation(path.join(IMGDIR,"splat.png"),64,1)
				new_anim.play(1)
				new_anim.set_fps(12)
				self.game.anims.append(new_anim)
				self.game.dead_pos = (self.rect.x - 16,self.rect.y - 16)
				self.game.game_sounds["over"].play()
				break
			
		
	def update(self,timepassed):
		
		if self.alive:
			self.move()
			self.delimit()
			self.handle_collisions()
			self.test_hit()
			
			self.dash_acctime += timepassed
			self.dash_wctime += timepassed
			
			if self.dash_acctime > self.max_dashtime:
				self.is_dashing = False
			
			self.prev_positions.append((self.rect.centerx,self.rect.centery))
			
			if len(self.prev_positions) > self.trail_len:
				self.prev_positions.pop(0)
			
			
			self.x += self.xvel
			self.y += self.yvel
			
			self.xvel *= self.friction
			self.yvel *= self.friction
			
			self.rect.topleft = (self.x,self.y)
			
			if True in [self.ml,self.mr,self.mu,self.md]:
				pass
			else:
				self.anim_id = 0
				self.anims[self.anim_id].play()
			
				
			for anim in self.anims:
				anim.update(timepassed)
			
			
			self.image = self.anims[self.anim_id].get_image()
			
			# calculate dist from center to player and mirror it for the cross_rect position
			xdist = self.game.g_center - self.x
			
			#self.cross_rect.x = self.game.g_center + xdist
			
			#self.emitter.update(timepassed)
			
			#calculate gun angle
			ang = math.atan2(self.cross_rect.y - self.y, self.cross_rect.x - self.x) * (180/math.pi)
			
			xvec = self.cross_rect.x - self.x
			yvec = self.cross_rect.y - self.y
			
			mag = math.sqrt(xvec*xvec + yvec*yvec)
			
			if mag == 0:
				mag = 0.00001
			
			xvec /= mag
			yvec /= mag
			
			xvec *= 7
			yvec *= 7
			
			self.gun_rect.centerx = self.rect.centerx + xvec
			self.gun_rect.centery = self.rect.centery + yvec
			
			self.gun.angle = int(ang)
			self.gun.x = self.rect.centerx 
			self.gun.y = self.rect.centery
			self.gun.update(timepassed)
			
			self.gun_image = pygame.transform.rotate(self.g_img,360 - int(ang))
			self.gun_rot_rect = self.gun_image.get_rect(center=self.gun_rect.center)
			
		
	def shoot(self):
		self.gun.fire()
	
class Ghost(object):
			
	def __init__(self,game,x,y):
		
		self.game = game
		self.x = x
		self.y = y
		
		self.xvel = 0
		self.yvel = 0
		
		self.emitter = Emitter(path.join(IMGDIR,"exp.png"),20,self.x,self.y,friction = 0.97,
									fps=12,angle_variance = 360, kind = 1,
												loop = 1,speed = 0,speed_variance=0)
	
	
		self.move_speeds = [0.75,0.95]
		self.speed_index = 0
		self.speed = self.move_speeds[self.speed_index] 
		
		self.rect = pygame.Rect(self.x,self.y,16,16)
		
		self.footprint_list = []
		self.change_speedtime = random.randint(5000,10000)
		self.acc_changetime = 0
		
		self.change_angletime = random.randint(1500,3000)
		self.acc_angtime = 0
		
		self.dir = 100 # get player self angle
		self.hp = 2
		
		self.alive = True
		
		stand_anim = Animation(path.join(IMGDIR,"green.png"),16,1)
		run_l_anim = Animation(path.join(IMGDIR,"green_runl.png"),16,1)
		run_r_anim = Animation(path.join(IMGDIR,"green_runr.png"),16,1)
		
		self.anims = [stand_anim,run_l_anim,run_r_anim]
		
		self.image = self.anims[0].get_image()
		
	def test_hit(self):
		
		for bullet in self.game.player.gun.emitter.particle_list:
			
			if bullet.rect.colliderect(self.rect):
				self.hp -= 1
				bullet.alive = False	
				self.emitter.burst(1)
				self.game.p_emitters.append(self.emitter)
				self.game.game_sounds["kill"].play()
				
				
	def update(self,timepassed):
		
		if self.hp <= 0:
			self.alive = False
		
		p = self.game.player
		
		self.acc_angtime += timepassed
		self.acc_changetime += timepassed
		
		if self.acc_angtime > self.change_angletime:
			
			xvec = p.x - self.x
			yvec = p.y - self.y
			
			self.dir = (math.atan2(yvec,xvec)) * (180/math.pi)
			self.acc_angtime = 0
			self.change_angletime = random.randint(1500,3000)
		
		if self.acc_changetime > self.change_speedtime:
			
			self.speed_index = (self.speed_index + 1) % len(self.move_speeds)
			self.speed = self.move_speeds[self.speed_index] 
			
			self.acc_changetime = 0
			self.change_speedtime = random.randint(5000,10000)
		
		self.test_hit()
		
		self.xvel = math.cos((self.dir * math.pi/180)) * self.speed
		self.yvel = math.sin((self.dir * math.pi/180)) * self.speed
		
		self.x += self.xvel
		self.y += self.yvel
		
		self.xvel *= 0.93
		self.yvel *= 0.93
		
		self.rect.topleft = (self.x,self.y)
		self.emitter.x = self.rect.centerx
		self.emitter.y = self.rect.centery
		
		
		if self.x < 0:
			self.x = 0
			self.xvel *= -1
		
		if self.x + self.rect.w > 320 :
			self.x = 320 - self.rect.w
			self.xvel *= -1
		
		if self.y < 0:
			self.y = 0
			self.yvel *= -1
		
		if self.y + self.rect.h > 240 :
			self.y = 240 - self.rect.h
			self.yvel *= -1
		
		for anim in self.anims:
			anim.update(timepassed)
		
		if self.xvel > 0:
			self.anims[2].play()
			self.image = self.anims[2].get_image()
		
		elif self.xvel < 0:
			self.anims[1].play()
			self.image = self.anims[1].get_image()
		
		else:
			self.image = self.anims[0].get_by_index(0)
			
			
	def draw(self,surface):
		
		for fp in self.footprint_list:
			fp.draw(surface)
		
		
		#pygame.draw.rect(surface,(0,100,200),self.rect)
		surface.blit(self.image,self.rect)
		
		
class GhostRed(Ghost):
	
	def __init__(self,game,x,y):
	
		super(GhostRed,self).__init__(game,x,y)
		
		self.move_speeds = [0.95,1.4]
		
		stand_anim = Animation(path.join(IMGDIR,"red.png"),16,1)
		run_l_anim = Animation(path.join(IMGDIR,"red_runl.png"),16,1)
		run_r_anim = Animation(path.join(IMGDIR,"red_runr.png"),16,1)
		
		self.anims = [stand_anim,run_l_anim,run_r_anim]
		
		self.image = self.anims[0].get_image()
		
		self.change_speedtime = random.randint(4000,6000)
		self.acc_changetime = 0
		
		self.change_angletime = random.randint(1000,2000)
		self.acc_angtime = 0
		self.hp = 3
		
	def update(self,timepassed):
		
		if self.hp <= 0:
			self.alive = False
		
		p = self.game.player
		
		self.acc_angtime += timepassed
		self.acc_changetime += timepassed
		
		if self.acc_angtime > self.change_angletime:
			
			xvec = p.x - self.x
			yvec = p.y - self.y
			
			self.dir = (math.atan2(yvec,xvec)) * (180/math.pi)
			self.acc_angtime = 0
			self.change_angletime = random.randint(1000,2000)
		
		if self.acc_changetime > self.change_speedtime:
			
			self.speed_index = (self.speed_index + 1) % len(self.move_speeds)
			self.speed = self.move_speeds[self.speed_index] 
			
			self.acc_changetime = 0
			self.change_speedtime = random.randint(4000,6000)
		
		self.test_hit()
		
		self.xvel = math.cos((self.dir * math.pi/180)) * self.speed
		self.yvel = math.sin((self.dir * math.pi/180)) * self.speed
		
		self.x += self.xvel
		self.y += self.yvel
		
		self.xvel *= 0.93
		self.yvel *= 0.93
		
		self.rect.topleft = (self.x,self.y)
		self.emitter.x = self.rect.centerx
		self.emitter.y = self.rect.centery
		
		
		if self.x < 0:
			self.x = 0
			self.xvel *= -1
		
		if self.x + self.rect.w > 320 :
			self.x = 320 - self.rect.w
			self.xvel *= -1
		
		if self.y < 0:
			self.y = 0
			self.yvel *= -1
		
		if self.y + self.rect.h > 240 :
			self.y = 240 - self.rect.h
			self.yvel *= -1
		
		for anim in self.anims:
			anim.update(timepassed)
		
		if self.xvel > 0:
			self.anims[2].play()
			self.image = self.anims[2].get_image()
		
		elif self.xvel < 0:
			self.anims[1].play()
			self.image = self.anims[1].get_image()
		
		else:
			self.image = self.anims[0].get_by_index(0)
			
class GhostBlue(Ghost):
	
	def __init__(self,game,x,y):
	
		super(GhostBlue,self).__init__(game,x,y)
		
		
		stand_anim = Animation(path.join(IMGDIR,"blue.png"),16,1)
		run_l_anim = Animation(path.join(IMGDIR,"blue_runl.png"),16,1)
		run_r_anim = Animation(path.join(IMGDIR,"blue_runr.png"),16,1)
		
		self.anims = [stand_anim,run_l_anim,run_r_anim]
		
		self.image = self.anims[0].get_image()
		
		self.xvel = 1.16
		self.yvel = (random.randint(1,10)) / 10.0
		
		
	def update(self,timepassed):
		
		if self.hp <= 0:
			self.alive = False
		
		p = self.game.player
		
		self.acc_angtime += timepassed
		self.acc_changetime += timepassed
		
		if self.acc_angtime > self.change_angletime:
			
			xvec = p.x - self.x
			yvec = p.y - self.y
			
			self.dir = (math.atan2(yvec,xvec)) * (180/math.pi)
			self.acc_angtime = 0
			self.change_angletime = random.randint(1000,2000)
		
		if self.acc_changetime > self.change_speedtime:
			
			self.speed_index = (self.speed_index + 1) % len(self.move_speeds)
			self.speed = self.move_speeds[self.speed_index] 
			
			self.acc_changetime = 0
			self.change_speedtime = random.randint(4000,6000)
		
		self.test_hit()
		
		r_dirs = [70,90]
		d = random.choice(r_dirs)
		
		
		self.x += self.xvel
		self.y += self.yvel
		
		#self.xvel *= 0.93
		#self.yvel *= 0.93
		
		self.rect.topleft = (self.x,self.y)
		self.emitter.x = self.rect.centerx
		self.emitter.y = self.rect.centery
		
		
		if self.x < 0:
			self.x = 0
			self.xvel *= -1
		
		if self.x + self.rect.w > 320 :
			self.x = 320 - self.rect.w
			self.xvel *= -1
		
		if self.y < 0:
			self.y = 0
			self.yvel *= -1
		
		if self.y + self.rect.h > 240 :
			self.y = 240 - self.rect.h
			self.yvel *= -1
		
		for anim in self.anims:
			anim.update(timepassed)
		
		if self.xvel > 0:
			self.anims[2].play()
			self.image = self.anims[2].get_image()
		
		elif self.xvel < 0:
			self.anims[1].play()
			self.image = self.anims[1].get_image()
		
		else:
			self.image = self.anims[0].get_by_index(0)
	

		
class Block(object):
			
	def __init__(self,x,y):
		
		self.x = x
		self.y = y
		
		self.image = pygame.image.load(path.join(IMGDIR,"block.png"))
		self.rect = self.image.get_rect()
		self.rect.topleft = (self.x,self.y)
		
	def draw(self, surface):
		
		surface.blit(self.image,self.rect)
				
	def update(self,timepassed):
		
		self.rect.topleft = (self.x,self.y)
		
class Spike(object):
	
	def __init__(self,x,y):
		
		self.x = x
		self.y = y
		
		self.state = 0 		# 0: off ; 1: on
		
		self.image_list = slice_sheet(32,32,path.join(IMGDIR,"trap.png"),1) 
		self.image = self.image_list[self.state]
		self.rect = self.image.get_rect()
		self.rect.topleft = (self.x,self.y)	
		
		self.state_updatetime = random.randint(3000,6000)
		self.state_acctime = 0
		
	def update(self,timepassed):
		
		self.state_acctime += timepassed
		if self.state_acctime > self.state_updatetime:
			
			self.state_acctime = 0
			self.state_updatetime = random.randint(3000,6000)
			
			if self.state == 0:
				self.state = 1
			else:
				self.state = 0
			
		self.image = self.image_list[self.state]
			
		
	def draw(self,surface):
		surface.blit(self.image,self.rect)
		
class Laser(object):
			
	def __init__(self,x,y,w,h,xvel,dir=0):
		
		self.x = x
		#self.y = random.randint(20,210)
		self.y = y
		self.dir = dir 		# 0 for left; 1 for right
		
		self.colors = [(255,0,0),(0,255,0),(255,255,255)]
		self.color_id = 0
		
		self.dcolor = self.colors[self.color_id]
		
		self.w = w
		self.h = h
		
		if self.dir == 0:
			self.xvel = -xvel
			self.anim = Animation(path.join(IMGDIR,"pusherl.png"),16,1)
		else:
			self.xvel = xvel
			self.anim = Animation(path.join(IMGDIR,"pusher.png"),16,1)
			
		self.anim.play()	
		self.rect = pygame.Rect(self.x,self.y,self.w,self.h)
		
		self.x += self.xvel
		self.rect.x = self.x
		self.alive = True
		
		self.image = self.anim.get_image()
		self.image_rect = self.image.get_rect()
		
	def update(self,timepassed):
		
		#self.color_id = (self.color_id + 1) % len(self.colors)
		self.dcolor = self.colors[self.color_id]
		
		self.x += self.xvel
		self.rect.x = self.x
		
		if self.dir == 0:
			if self.x < -20:
				self.alive = False
			
			self.image_rect.midleft = (self.rect.right,self.rect.centery)
		
		if self.dir == 1:
			if self.x > 340:
				self.alive = False
			
			self.image_rect.midright = (self.rect.left,self.rect.centery)
		
		self.anim.update(timepassed)
		
	def draw(self,surface):
		
		pygame.draw.rect(surface,self.dcolor,self.rect)
		
		self.image = self.anim.get_image()
		surface.blit(self.image,self.image_rect)
		
		
		