import pygame, sys
from pygame.locals import*
from entity import*


class Game(object):
	
	def __init__(self, full):
		
		pygame.mixer.pre_init(44100,-16,2,1024)
		pygame.init()
		
		if full:
			self.screen = pygame.display.set_mode((320,240),DOUBLEBUF|FULLSCREEN)
		else:
			self.screen = pygame.display.set_mode((320,240),DOUBLEBUF)
			
		pygame.mouse.set_visible(False)	
		self.font = pygame.font.Font(path.join(FNTDIR,"Minecraftia-Regular.ttf"),25)
		self.sfont = pygame.font.Font(path.join(FNTDIR,"Minecraftia-Regular.ttf"),13)
		
		self.bg = pygame.image.load(path.join(IMGDIR,"bg.png")).convert_alpha()
		self.game_sounds = {}
		self.game_sounds["fire"] = pygame.mixer.Sound(path.join(SNDDIR,"fire.ogg"))
		self.game_sounds["kill"] = pygame.mixer.Sound(path.join(SNDDIR,"kill.ogg"))
		self.game_sounds["over"] = pygame.mixer.Sound(path.join(SNDDIR,"over.ogg"))
		self.game_sounds["dash"] = pygame.mixer.Sound(path.join(SNDDIR,"dash.ogg"))
		self.game_sounds["win"] = pygame.mixer.Sound(path.join(SNDDIR,"win.ogg"))
		self.game_sounds["pick"] = pygame.mixer.Sound(path.join(SNDDIR,"pick.ogg"))
		self.game_sounds["dead"] = pygame.mixer.Sound(path.join(SNDDIR,"dead.ogg"))
		
		self.score = 0
		self.point_val = 50
		
		
		self.difficulty = 1
		self.d_surf = pygame.Surface((320,240))
		self.d_surf_rect = self.d_surf.get_rect()
		self.d_surf_rect.topleft = (0,0)
		self.dead_pos = (0,0)
		
		pygame.mixer.music.load(path.join(SNDDIR,"theme.ogg"))
		
		self.reset()
		self.get_diff_states(int(self.difficulty))
		self.create_level()
		
	def get_diff_states(self, difficulty_level):
		
		self.max_enemies = int(self.difficulty * 1.5)
		
		#if difficulty_level in range(4):
		#	self.max_enemies = 3
		#	self.add_laser_time = 8000
		
		#if difficulty_level in range(4,6):
		#	self.max_enemies = 6
		#	self.add_laser_time = 5000
		
		#if difficulty_level in range(6,8):
		#	self.max_enemies = 8
		#	self.add_laser_time = 4000
		
		#if difficulty_level in range(8,11):
		#	self.max_enemies = 8
		#	self.add_laser_time = 4000
		
		self.add_laser_time = max(3000,8000 - (200 * self.difficulty))
		
	
	def show_pause(self):
		
		esc_txt = self.sfont.render("press esc to return to game",False,(200,200,200))
		quit_text = self.sfont.render("press q to quit",False,(200,200,200))
		
		while 1:
			
			for event in pygame.event.get():
				if event.type == QUIT:
					pygame.quit()
					sys.exit()
					
				if event.type == KEYDOWN:
					if event.key == K_ESCAPE:
						return 0
						
					if event.key == K_q:
						return 1
						
			self.d_surf.fill((0,0,0))		
			
			self.d_surf.blit(esc_txt, (50,100))
			self.d_surf.blit(quit_text,(50,130))
			
			self.screen.blit(self.d_surf,self.d_surf_rect)
			
			pygame.display.flip()
			
	
	def show_start(self):
		
		pygame.mixer.music.play(-1)
		
		esc_txt = self.sfont.render("press 'Z' to shoot and 'X' to dash",False,(200,200,200))
		quit_text = self.sfont.render("press esc to quit",False,(200,200,200))
		
		banner_anim = Animation(path.join(IMGDIR,"banner.png"),128,1)
		banner_anim.play()
		banner_anim.set_fps(6)
		
		clock = pygame.time.Clock()
		timepassed = 0
		
		while 1:
			
			for event in pygame.event.get():
				if event.type == QUIT:
					pygame.quit()
					sys.exit()
					
				if event.type == KEYDOWN:
					if event.key == K_ESCAPE:
						pygame.quit()
						sys.exit()
					
					if event.key in [K_RETURN,K_z,K_x]:
						return
			
			banner_anim.update(timepassed)			
							
			self.d_surf.fill((0,0,0))		
			
			img = banner_anim.get_image()
			self.d_surf.blit(img, (40,30))
			
			self.d_surf.blit(esc_txt, (50,100))
			self.d_surf.blit(quit_text,(50,130))
			
			self.screen.blit(self.d_surf,self.d_surf_rect)
			
			pygame.display.flip()
			
			timepassed = clock.tick(60)
	
	
	def show_gameover(self):
		
		dead_txt = self.font.render("game over",False,(200,200,200))
		dead_rect = dead_txt.get_rect()
		dead_rect.center = (320/2,240/2)
		
		score_txt = self.font.render("your score: %s" % self.score,False,(200,200,200))
		score_rect = score_txt.get_rect()
		score_rect.center = (310/2,160)
		
		
		while 1:
			
			for event in pygame.event.get():
				if event.type == QUIT:
					pygame.quit()
					sys.exit()
					
				if event.type == KEYDOWN:
					if event.key in [K_RETURN,K_x,K_z]:
						return 1
						
					if event.key == K_ESCAPE:
						return 0
						
			self.screen.fill((0,0,0))		
			
			
			self.screen.blit(dead_txt, dead_rect)
			self.screen.blit(score_txt, score_rect)
			
			pygame.display.flip()
			
			
	def create_level(self):
		
		self.level_surf = pygame.Surface((320,640))
		self.level_surf.set_colorkey((255,0,255))
		
		x_positions = range(320/32)
		y_positions = range(240/32)
		
		spawn_positions = [(30,30),(130,30),(290,30),(30,200),(130,200),(290,200),(30,150),(30,80),(30,200),
							(280,150),(280,80),(290,20)]
		
		"""
		
		for i in range(self.num_traps):
			new_x = random.choice(x_positions) * 32
			new_y = random.choice(y_positions) * 32
				
			new_trap = Spike(new_x,new_y)
			self.traps.append(new_trap)
			
		for i in range(0,320,32):
		#for i in range(1):
			
			# create block
			new_block = Block(i,self.block_y)
			self.block_objects.append(new_block)
		
		"""
		
		for i in range(self.max_enemies):
			pos = random.choice(spawn_positions)
				
			new_trap = random.choice([Ghost,GhostRed,GhostBlue])(self,pos[0],pos[1])
			self.traps.append(new_trap)
		
		#print len(self.traps)
		
		
	def reset(self):
		
		self.exit_i = 0				# 0 - you died  , 1 - you won
		self.block_objects = []
		self.block_y = 60
		self.g_center = 320/2
		self.num_traps = 5
		self.traps = []
		self.lasers = []
		self.anims = []
		
		self.p_emitters = []
		self.to_menu = False
		
		self.is_running = True
		self.level_surf = None
		
		# game objects and stuff
		self.player = Player(self,150,120)
		self.mx = self.my = 0
		
		self.max_enemies = 6
		
		# laser frequency
		self.add_laserfreq = 0.0
		self.add_laser_time = 5000
		self.laser_acctime = 0
		self.fp = 0.0001
		self.laser_speed = 1.4
		
		self.laser_updatetime = max(1 - self.add_laserfreq,0) * self.add_laser_time 
			
	
	def laser_config1(self):
		
		new_laser = Laser(-20,0,10,80,self.laser_speed,dir=1)
		new_laser2 = Laser(340,160,10,80,self.laser_speed,dir=0)
		
		self.lasers.append(new_laser)
		self.lasers.append(new_laser2)
	
	def laser_config2(self):
		
		new_laser = Laser(-20,0,10,50,self.laser_speed,dir=1)
		new_laser2 = Laser(340,100,10,50,self.laser_speed,dir=0)
		new_laser3 = Laser(-20,180,10,50,self.laser_speed,dir=1)
		
		self.lasers.append(new_laser)
		self.lasers.append(new_laser2)
		self.lasers.append(new_laser3)
	
	def laser_config3(self):
		
		new_laser = Laser(340,0,10,140,self.laser_speed,dir=0)
		self.lasers.append(new_laser)
		
	def laser_config4(self):
		
		new_laser = Laser(-20,80,10,140,self.laser_speed,dir=1)
		self.lasers.append(new_laser)
		
	def laser_config5(self):
		
		new_laser = Laser(340,80,10,140,self.laser_speed,dir=0)
		self.lasers.append(new_laser)
		
	def laser_config6(self):
		
		new_laser = Laser(-20,80,10,140,self.laser_speed,dir=1)
		new_laser2 = Laser(340,0,10,140,self.laser_speed,dir=0)
		
		self.lasers.append(new_laser)
		self.lasers.append(new_laser2)
	
	def laser_config7(self):
		
		new_laser = Laser(-20,0,10,70,self.laser_speed,dir=1)
		new_laser2 = Laser(-120,80,10,70,self.laser_speed,dir=1)
		new_laser3 = Laser(-240,150,10,70,self.laser_speed,dir=1)
		
		self.lasers.append(new_laser)
		self.lasers.append(new_laser2)
		self.lasers.append(new_laser3)
	
	def add_laser(self):
		"""
		num = random.choice([0,1])
		
		if num:
			ndir = num
			xpos = -20
		else:
			ndir = num
			xpos = 340
		
		new_laser = Laser(xpos,self.laser_speed,dir=ndir)
		self.lasers.append(new_laser)
		"""
		funcs = [self.laser_config1, self.laser_config2,self.laser_config3,self.laser_config4,
				self.laser_config5,self.laser_config6,self.laser_config7]
		f = random.choice(funcs)
		
		f()
		
		
		
	def update(self,timepassed):
		
		self.add_laserfreq += self.fp 
		self.laser_acctime += timepassed
		
		if self.laser_acctime > self.laser_updatetime:
			self.laser_acctime = 0
			self.laser_updatetime = max(1 - self.add_laserfreq,0.5) * self.add_laser_time 
			self.add_laser()
		
		if len(self.traps) < 1:
			self.is_running = False
			self.exit_i = 1
			self.to_menu = True
			#print "hello"
			self.difficulty += 1
			pygame.time.wait(1000)
				
			
		#print self.difficulty
		#print self.laser_updatetime
		#print self.add_laserfreq
		#print len(self.lasers)
		
		self.player.update(timepassed)
		
		for trap in self.traps:
			trap.update(timepassed)
		
		for laser in self.lasers:
			laser.update(timepassed)
		
		for emitter in self.p_emitters:
			emitter.update(timepassed)
		
		for laser in self.lasers:
			if not laser.alive:
				self.lasers.remove(laser)
		
		for trap in self.traps:
			if not trap.alive:
				self.traps.remove(trap)
				self.score += self.point_val
				self.game_sounds["dead"].play()
		
		for emitter in self.p_emitters:
			if len(emitter.particle_list) < 1:
				self.p_emitters.remove(emitter)
		
		for item in self.anims:
			item.update(timepassed)
			
		for item in self.anims:
			if item.is_ended:
				self.is_running = False
				pygame.time.wait(1000)
		
		
	def draw(self):
		
		self.screen.fill((0,0,0))
		
		self.d_surf.fill((0,0,0))
		self.level_surf.fill((0,0,0))
		self.level_surf.blit(self.bg,(0,0))
		
		
		for block in self.block_objects:
			block.draw(self.level_surf)
		
		for trap in self.traps:
			trap.draw(self.level_surf)
		
		for laser in self.lasers:
			laser.draw(self.level_surf)
		
		for emitter in self.p_emitters:
			emitter.draw(self.level_surf)
		
		self.player.draw(self.level_surf)
		
		if len(self.anims)>0:
			img = self.anims[0].get_image()
			self.level_surf.blit(img,self.dead_pos)
			
		
		self.d_surf.blit(self.level_surf,(0,0))
		
		score_txt = self.sfont.render("score : %s" % self.score,False,(255,255,255))
		self.d_surf.blit(score_txt, (20,10))
		
		
		self.screen.blit(self.d_surf,self.d_surf_rect)
		
	def run(self):
		
		clock = pygame.time.Clock()
		timepassed = 0
		
		pygame.mixer.music.stop()
		
		while self.is_running:
			
			self.handle_events()
			self.update(timepassed)
			self.draw()
			
			pygame.display.flip()
			
			timepassed = clock.tick()
		
	def handle_events(self):
		
		for event in pygame.event.get():
			if event.type == QUIT:
				pygame.quit()
				sys.exit()
				
			if event.type == KEYDOWN:
				if event.key == K_ESCAPE:
					#pygame.quit()
					#sys.exit()
					result = self.show_pause()
					if result == 1:
						# exit to menu
						self.is_running = False
						self.to_menu = True
						
				if event.key == K_z:
					self.player.shoot()
			
				if event.key == K_x:
					self.player.dash()
			
			
			if event.type == MOUSEMOTION:
				
				self.mx, self.my = event.pos
				
				#self.player.set_cursor_pos(self.mx,self.my)
				
		pressed = pygame.key.get_pressed()

		if pressed[K_LEFT]:
			self.player.ml = True
		else:
			self.player.ml = False
		
		if pressed[K_RIGHT]:
			self.player.mr = True
		else:
			self.player.mr = False
					
		if pressed[K_UP]:
			self.player.mu = True
		else:
			self.player.mu = False
		
		if pressed[K_DOWN]:
			self.player.md = True
		else:
			self.player.md = False
		
	
	

