from animate import*
import pygame, sys, math, random

class Particle(object):
	
	def __init__(self,filename,width,x,y,**kwargs):
		
		self.x = x
		self.y = y
		self.alive = True
		self.acctime = 0
		
		valid_kwargs = {
			
			"kind" : 0,
			"xvel" : 0,
			"yvel" : 0,
			"fps" : 12,
			"friction" : 1.0,
			"gravity" : 0.0,
			"loop" : 1,
			"lifetime" : None		
		}
		
		for key,value in kwargs.items():
			if valid_kwargs.has_key(key):
				valid_kwargs[key] = value
				
		vars(self).update(valid_kwargs)		
			
		self.animator = Animation(filename,width,self.kind)
		self.animator.set_fps(self.fps)
		
		self.max_cycles = len(self.animator.image_list) * self.loop
		
		self.image = self.animator.get_image()
		self.rect = self.image.get_rect()
		self.rect.center = (self.x,self.y)
		
		self.animator.start()
		
	def update(self,timepassed):
		
		self.animator.update(timepassed)
		self.acctime += timepassed
		
		if self.loop > 0:
			if self.animator.cycles >= self.max_cycles:
				
				#self.animator.show_last()
				self.alive = False
		
		if self.lifetime is not None:
			if self.acctime > self.lifetime:
				self.alive = False
		
		if self.alive:
			self.image = self.animator.get_image()
			
			self.yvel += self.gravity
			self.x += self.xvel
			self.y += self.yvel
			
			self.rect.center = (self.x,self.y)
			
			self.xvel *= self.friction
			self.yvel *= self.friction
			
	def draw(self,surface):
		
		surface.blit(self.image,self.rect)
	
class Emitter(object):
	
	def __init__(self,filename,width,x,y,**kwargs):
		
		self.filename = filename
		self.width = width
		self.x = x
		self.y = y
		self.particle_list = []
		self.acctime = 0
		self.emit = False
		
		self.valid_kwargs = {
			
			"kind" : 0,
			"angle" : 0,
			"angle_variance": 0,
			"speed" : 1.5,
			"speed_variance": 0.0,
			"fps" : 6,
			"fps_variance" : 0,
			"friction" : 1.0,
			"gravity" : 0.0,
			"updatetime" : 200,
			"loop" : 1,
			"lifetime" : None		
		}
		
		for key,value in kwargs.items():
			if self.valid_kwargs.has_key(key):
				
				self.valid_kwargs[key] = value
				
		vars(self).update(self.valid_kwargs)		
		
		#self.start_stream()
		
	def start_stream(self):
		self.emit = True
	
	def stop_stream(self):
		self.emit = False
	
	def burst(self,num_particles):	
		
		for i in range(num_particles):
			self.add_particle()
		
	def add_particle(self):
		
		angle = random.randint(self.angle - self.angle_variance, self.angle + self.angle_variance)
		speed = random.uniform(max(self.speed - self.speed_variance,0),self.speed + self.speed_variance )
		new_fps = max(random.randint(self.fps - self.fps_variance, self.fps + self.fps_variance),1)
		
		new_xvel = math.cos((angle)*math.pi/180) * speed
		new_yvel = math.sin((angle)*math.pi/180) * speed
		
		
		new_particle = Particle(self.filename,self.width,self.x,self.y,
									xvel = new_xvel,
									yvel = new_yvel,
									friction = self.friction,
									gravity = self.gravity,
									fps = new_fps,
									loop = self.loop,
									kind = self.kind,
									lifetime = self.lifetime)
									
		self.particle_list.append(new_particle)
				
		
	def update(self,timepassed):
		
		if self.emit:
			self.acctime += timepassed
			if self.acctime > self.updatetime:
				
				self.add_particle()	
				self.acctime = 0
			
		for particle in self.particle_list:
			if not particle.alive:
				self.particle_list.remove(particle)
		
		for particle in self.particle_list:
			particle.update(timepassed)
		
	def draw(self,surface):
		
		for particle in self.particle_list:
			#if particle.alive:
			particle.draw(surface)
